//  COPYRIGHT:       PrimeObjects Software Inc. (C) 2022 All Right Reserved
//  COMPANY URL:     https://www.primeobjects.com/
//  CONTACT:         developer@primeobjects.com
//
//  This source is subject to the PrimeObjects License Agreements.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import React, { useState, useEffect } from "react";
import PropTypes from "prop-types";
import { getWindow, isSSR } from "primeobjects-helper-util/build/cjs/core";
import CSS from "primeobjects-ui-web-store/build/cjs/css";
import ResizeDetectorDiv from "./resize-detector-div";
var SPLITTER_CSS = "\n.splitter-layout {\n    display: flex;\n    flex-direction: row;\n    width: 100%;\n    height: 100%;\n    overflow: hidden;\n  }\n  \n  .splitter-layout .layout-pane {\n    position: relative;\n    flex: 0 0 auto;\n    overflow: auto;\n  }\n  \n  .splitter-layout .layout-pane.layout-pane-primary {\n    flex: 1 1 auto;\n  }\n  \n  .splitter-layout > .layout-splitter {\n    flex: 0 0 auto;\n    width: 4px;\n    cursor: col-resize;\n    background-color: transparent;\n  }\n  \n  .splitter-layout .layout-splitter:hover {\n    background-color: transparent;\n  }\n  \n  .splitter-layout.layout-changing {\n    cursor: col-resize;\n  }\n  \n  .splitter-layout.layout-changing > .layout-splitter {\n    background-color: transparent;\n  }\n  \n  .splitter-layout.splitter-layout-vertical {\n    flex-direction: column;\n  }\n  \n  .splitter-layout.splitter-layout-vertical.layout-changing {\n    cursor: row-resize;\n  }\n  \n  .splitter-layout.splitter-layout-vertical > .layout-splitter {\n    width: 100%;\n    height: 4px;\n    cursor: row-resize;\n  }\n  ";
function Pane(props) {
    var size = props.size || 0;
    var unit = props.percentage ? "%" : "px";
    var classes = "layout-pane";
    var style = {};
    if (!props.primary) {
        if (props.vertical) {
            style.height = "".concat(size).concat(unit);
        }
        else {
            style.width = "".concat(size).concat(unit);
        }
    }
    else {
        classes += " layout-pane-primary";
    }
    return (React.createElement("div", { className: classes, style: style }, props.children));
}
Pane.propTypes = {
    vertical: PropTypes.bool,
    primary: PropTypes.bool,
    size: PropTypes.number,
    percentage: PropTypes.bool,
    children: PropTypes.oneOfType([
        PropTypes.arrayOf(PropTypes.node),
        PropTypes.node
    ])
};
Pane.defaultProps = {
    vertical: false,
    primary: false,
    size: 0,
    percentage: false,
    children: []
};
function clearSelection() {
    var win = getWindow();
    var doc = win.document ? win.document : {};
    if (doc.body.createTextRange) {
        // https://github.com/zesik/react-splitter-layout/issues/16
        // https://stackoverflow.com/questions/22914075/#37580789
        var range = doc.body.createTextRange();
        range.collapse();
        range.select();
    }
    else if (win.getSelection) {
        if (win.getSelection().empty) {
            win.getSelection().empty();
        }
        else if (win.getSelection().removeAllRanges) {
            win.getSelection().removeAllRanges();
        }
    }
    else if (doc.selection) {
        doc.selection.empty();
    }
}
var DEFAULT_SPLITTER_SIZE = 4;
var SplitterLayout = /** @class */ (function (_super) {
    __extends(SplitterLayout, _super);
    function SplitterLayout(props) {
        var _this = _super.call(this, props) || this;
        _this.splitter = React.createRef();
        _this.container = React.createRef();
        _this.handleResize = _this.handleResize.bind(_this);
        _this.handleMouseMove = _this.handleMouseMove.bind(_this);
        _this.handleMouseUp = _this.handleMouseUp.bind(_this);
        _this.handleTouchMove = _this.handleTouchMove.bind(_this);
        _this.handleSplitterMouseDown = _this.handleSplitterMouseDown.bind(_this);
        _this.state = {
            secondaryPaneSize: 0,
            resizing: false
        };
        return _this;
    }
    SplitterLayout.prototype.componentDidMount = function () {
        var _a;
        var win = getWindow();
        var doc = win.document ? win.document : {};
        win.addEventListener("resize", this.handleResize);
        doc.addEventListener("mouseup", this.handleMouseUp);
        doc.addEventListener("mousemove", this.handleMouseMove);
        doc.addEventListener("touchend", this.handleMouseUp);
        doc.addEventListener("touchmove", this.handleTouchMove);
        var secondaryPaneSize;
        if (typeof this.props.secondaryInitialSize !== "undefined") {
            secondaryPaneSize = this.props.secondaryInitialSize;
        }
        else {
            var containerRect = this.container.current.getBoundingClientRect();
            var splitterRect = void 0;
            if ((_a = this.splitter) === null || _a === void 0 ? void 0 : _a.current) {
                splitterRect = this.splitter.current.getBoundingClientRect();
            }
            else {
                // Simulate a splitter
                splitterRect = {
                    width: DEFAULT_SPLITTER_SIZE,
                    height: DEFAULT_SPLITTER_SIZE
                };
            }
            secondaryPaneSize = this.getSecondaryPaneSize(containerRect, splitterRect, {
                left: containerRect.left +
                    (containerRect.width - splitterRect.width) / 2,
                top: containerRect.top +
                    (containerRect.height - splitterRect.height) / 2
            }, false);
        }
        this.setState({ secondaryPaneSize: secondaryPaneSize });
    };
    SplitterLayout.prototype.componentDidUpdate = function (prevProps, prevState) {
        if (prevProps)
            prevState;
        if (prevState.secondaryPaneSize !== this.state.secondaryPaneSize &&
            this.props.onSecondaryPaneSizeChange) {
            this.props.onSecondaryPaneSizeChange(this.state.secondaryPaneSize);
        }
        if (prevState.resizing !== this.state.resizing) {
            if (this.state.resizing) {
                if (this.props.onDragStart) {
                    this.props.onDragStart();
                }
            }
            else if (this.props.onDragEnd) {
                this.props.onDragEnd();
            }
        }
    };
    SplitterLayout.prototype.componentWillUnmount = function () {
        var win = getWindow();
        var doc = win.document ? win.document : {};
        win.removeEventListener("resize", this.handleResize);
        doc.removeEventListener("mouseup", this.handleMouseUp);
        doc.removeEventListener("mousemove", this.handleMouseMove);
        doc.removeEventListener("touchend", this.handleMouseUp);
        doc.removeEventListener("touchmove", this.handleTouchMove);
    };
    SplitterLayout.prototype.getSecondaryPaneSize = function (containerRect, splitterRect, clientPosition, offsetMouse) {
        var totalSize;
        var splitterSize;
        var offset;
        if (this.props.vertical) {
            totalSize = containerRect.height;
            splitterSize = splitterRect.height;
            offset = clientPosition.top - containerRect.top;
        }
        else {
            totalSize = containerRect.width;
            splitterSize = splitterRect.width;
            offset = clientPosition.left - containerRect.left;
        }
        if (offsetMouse) {
            offset -= splitterSize / 2;
        }
        if (offset < 0) {
            offset = 0;
        }
        else if (offset > totalSize - splitterSize) {
            offset = totalSize - splitterSize;
        }
        var secondaryPaneSize;
        if (this.props.primaryIndex === 1) {
            secondaryPaneSize = offset;
        }
        else {
            secondaryPaneSize = totalSize - splitterSize - offset;
        }
        var primaryPaneSize = totalSize - splitterSize - secondaryPaneSize;
        if (this.props.percentage) {
            secondaryPaneSize = (secondaryPaneSize * 100) / totalSize;
            primaryPaneSize = (primaryPaneSize * 100) / totalSize;
            splitterSize = (splitterSize * 100) / totalSize;
            totalSize = 100;
        }
        if (primaryPaneSize < this.props.primaryMinSize) {
            secondaryPaneSize = Math.max(secondaryPaneSize -
                (this.props.primaryMinSize - primaryPaneSize), 0);
        }
        else if (secondaryPaneSize < this.props.secondaryMinSize) {
            secondaryPaneSize = Math.min(totalSize - splitterSize - this.props.primaryMinSize, this.props.secondaryMinSize);
        }
        return secondaryPaneSize;
    };
    SplitterLayout.prototype.handleResize = function () {
        if (this.splitter.current && !this.props.percentage) {
            var containerRect = this.container.current.getBoundingClientRect();
            var splitterRect = this.splitter.current.getBoundingClientRect();
            var secondaryPaneSize = this.getSecondaryPaneSize(containerRect, splitterRect, {
                left: splitterRect.left,
                top: splitterRect.top
            }, false);
            this.setState({ secondaryPaneSize: secondaryPaneSize });
        }
    };
    SplitterLayout.prototype.handleMouseMove = function (e) {
        if (this.state.resizing) {
            var containerRect = this.container.current.getBoundingClientRect();
            var splitterRect = this.splitter.current.getBoundingClientRect();
            var secondaryPaneSize = this.getSecondaryPaneSize(containerRect, splitterRect, {
                left: e.clientX,
                top: e.clientY
            }, true);
            clearSelection();
            this.setState({ secondaryPaneSize: secondaryPaneSize });
        }
    };
    SplitterLayout.prototype.handleTouchMove = function (e) {
        this.handleMouseMove(e.changedTouches[0]);
    };
    SplitterLayout.prototype.handleSplitterMouseDown = function () {
        clearSelection();
        this.setState({ resizing: true });
    };
    SplitterLayout.prototype.handleMouseUp = function () {
        this.setState(function (prevState) {
            return prevState.resizing ? { resizing: false } : null;
        });
    };
    SplitterLayout.prototype.render = function () {
        var containerClasses = "splitter-layout";
        if (this.props.customClassName) {
            containerClasses += " ".concat(this.props.customClassName);
        }
        if (this.props.vertical) {
            containerClasses += " splitter-layout-vertical";
        }
        if (this.state.resizing) {
            containerClasses += " layout-changing";
        }
        var children = React.Children.toArray(this.props.children).slice(0, 2);
        if (children.length === 0) {
            children.push(React.createElement("div", null));
        }
        var wrappedChildren = [];
        var primaryIndex = this.props.primaryIndex !== 0 && this.props.primaryIndex !== 1
            ? 0
            : this.props.primaryIndex;
        for (var i = 0; i < children.length; ++i) {
            var primary = true;
            var size = null;
            if (children.length > 1 && i !== primaryIndex) {
                primary = false;
                size = this.state.secondaryPaneSize;
            }
            wrappedChildren.push(React.createElement(Pane, { vertical: this.props.vertical, percentage: this.props.percentage, primary: primary, size: size }, children[i]));
        }
        return (React.createElement("div", { className: containerClasses, ref: this.container },
            wrappedChildren[0],
            wrappedChildren.length > 1 && (React.createElement("div", { role: "separator", className: "layout-splitter", ref: this.splitter, onMouseDown: this.handleSplitterMouseDown, onTouchStart: this.handleSplitterMouseDown })),
            wrappedChildren.length > 1 && wrappedChildren[1]));
    };
    return SplitterLayout;
}(React.Component));
export var HorizontalSplitter = function (props) {
    var _a = useState(false), show = _a[0], setShow = _a[1];
    var _b = useState(0), secondaryInitialSize = _b[0], setSecondaryInitialSize = _b[1];
    var SSR = isSSR();
    useEffect(function () {
        setShow(true);
    }, [SSR]);
    var onResize = function (width) {
        if (props.leftPaneSize) {
            setSecondaryInitialSize(props.leftPaneSize ? width - props.leftPaneSize : width * 0.5);
        }
        else {
            setSecondaryInitialSize(props.rightPaneSize ? props.rightPaneSize : width * 0.5);
        }
    };
    return (React.createElement(ResizeDetectorDiv, { className: "w-full h-full", onDetect: onResize },
        React.createElement(CSS, { id: "splitter-css", content: SPLITTER_CSS }),
        show && secondaryInitialSize > 0 && (React.createElement(SplitterLayout, { secondaryInitialSize: secondaryInitialSize, primaryMinSize: props.leftPaneMinSize, secondaryMinSize: props.rightPaneMinSize }, props.children))));
};
export default HorizontalSplitter;
//# sourceMappingURL=horizontal-splitter.js.map